#include <unistd.h>
#include <stdio.h>
#include <stdbool.h>
#include <string.h>
#include <math.h>
#include <errno.h>
#include <MSXE351x.nsmap> // this file must be included only once in the project
#include <assert.h>
#include <sys/stat.h>
#include <signal.h>

#include "utils.h"
/* please see utils.h */
int DUMMY = 0;

//-----------------------------------------------------------------------------------------
/** writeMoreFunction : write values on the analog output of the MSX-E351x
 *
 * @param[in] soapEnv The current SOAP environment
 * @param[in] endPoint : SOAP endpoint
 * @param[in] nbrOfChannel : Number of channel on the board
 *
 * @return 0
 *
 */
int writeMoreFunction(struct soap *soapenv, char *endpoint, unsigned long nbrOfChannel)
{
    struct MSXE351x__Response response;
    struct MSXE351x__AnalogOutputGetStatusResponse statusResponse;
    unsigned long channelMask = 0;
    struct MSXE351x__unsignedLong8FixedArrayParam outputType;
    struct MSXE351x__unsignedLong8FixedArrayParam polarity;
    unsigned long triggerMask = 0;
    unsigned long triggerEdgeSelection = 0;
    unsigned long triggerCount = 0;
    struct MSXE351x__unsignedLong8FixedArrayParam channelValue;
    struct MSXE351x__unsignedLong8FixedArrayParam pulDummy;
    unsigned long cpt = 0;

    printf("\n");

    do
    {
        printf("\nEnter the channel mask do You want to write in Hex (1 to 0x%lx) : 0x", (unsigned long)((1 << nbrOfChannel) - 1));
        scanf("%lX", &channelMask);
    } while ((channelMask == 0) || (channelMask > (1 << nbrOfChannel) - 1));

    for (cpt = 0; cpt < nbrOfChannel; cpt++)
    {
        if (channelMask & (1 << cpt))
        {
            printf("\nEnter the information for the channel %lu :", cpt);
            do
            {
                printf("\nEnter the channel configuration do You want to use (0:Voltage, 1:Current) :");
                scanf("%lu", &outputType.ulValue[cpt]);
            } while (outputType.ulValue[cpt] > 1);
            do
            {
                printf("\nEnter the channel polarity do You want to use (0:Unipolar, 1:Bipolar) :");
                scanf("%lu", &polarity.ulValue[cpt]);
            } while (polarity.ulValue[cpt] > 1);

            do
            {
                if (polarity.ulValue[cpt] == 0)
                    printf("\nEnter the output value to write in Hex (0 to 0x7FFF) : 0x");
                else
                    printf("\nEnter the output value to write in Hex (0 to 0xFFFF) : 0x");

                scanf("%lX", &channelValue.ulValue[cpt]);
            } while (((channelValue.ulValue[cpt] > 0xFFFF) && (polarity.ulValue[cpt] == 1)) || ((channelValue.ulValue[cpt] > 0x7FFF) && (polarity.ulValue[cpt] == 0)));
        }
    }

    do
    {
        printf("\nDo You want to use an external trigger ? (0:No trigger used, 1:Digital Input Trigger, 2:Synchro Trigger) :");
        scanf("%lu", &triggerMask);
    } while (triggerMask > 2);

    if (triggerMask == 1)
    {
        do
        {
            printf("\nSelect the edge to react for the digital input trigger (1:rising edge, 2:falling edge, 3:both edge) :");
            scanf("%lu", &triggerEdgeSelection);
        } while ((triggerEdgeSelection == 0) || (triggerEdgeSelection > 3));
        do
        {
            printf("\nSelect the digital input trigger count value (min 1):");
            scanf("%lu", &triggerCount);
        } while (triggerCount == 0);

    }

    assert_gsoap(soapenv, soap_call_MSXE351x__AnalogOutputWriteMoreValues(soapenv, endpoint, NULL, channelMask, &outputType, &polarity, triggerMask, triggerEdgeSelection, triggerCount, &channelValue, 0, 0, 0, &pulDummy, &response));
    assert_soap_command(soapenv, (struct DefaultResponse *)&response);

    if (triggerMask == 0)
        printf("\nThe values have been written to the output !");
    else
    {
        printf("\n Waiting for a trigger to write the values! Press ESC to abort...");

        for (;;)
        {
            assert_gsoap(soapenv, soap_call_MSXE351x__AnalogOutputGetStatus(soapenv, endpoint, NULL, 0, &statusResponse));
            assert_soap_command(soapenv, (struct DefaultResponse *)&response);

            /* Release the memory allocated by the gSOAP calls */
            soap_destroy(soapenv);
            soap_end(soapenv);

            if (statusResponse.ulStatus == 0)
            {
                printf("\n The trigger occurred, the outputs are ready to receive another value.");
                return 0;
            }
            else if (kbhit() && getch() == 27)
            {
                return 1;
            }
        }
    }
    return 0;
}

//-------------------------------------------------------------------------------------------------------

int main(int argc, char ** argv)
{
    char endpoint[30];
    struct soap soapenv;
    struct MXCommon__ByteArrayResponse response;
    int nbrOfChannel = 8;
    int DUMMY = 0;

    /* Print sample information */
    printf("+---------------------------------------------------+\n");
    printf("| MSX-E351x                                         |\n");
    printf("| Analog output write more values sample            |\n");
    printf("+---------------------------------------------------+\n");
    printf("\n");

    if (argc != 2)
    {
        printf("\nThe command should be like this : ./WriteOne XXX.XXX.XXX.XXX\n\n");
        exit(1);
    }

    memset(endpoint, 0, sizeof(endpoint));

    /* this structure the current SOAP connection */
    mx_soap_init(&soapenv);

    /* generates the endpoint string
     * SOAP "endpoints" are of the form NAME:PORT -
     * PORT is in the case of the SOAP command server: 5555
     */
    if (snprintf(endpoint, sizeof(endpoint), "%s:5555", argv[1]) < 0)
    {
        fprintf(stderr, "error: %s:%d!\n", __FUNCTION__, __LINE__);
        exit(1);
    }

    /* ask the module for its type */

    assert_gsoap(&soapenv, soap_call_MXCommon__GetModuleType(&soapenv, endpoint, NULL, &DUMMY, &response) );
    printf("Returned string: %s\n", response.sArray.__ptr);

    /* who is currently connected ? */
    assert_gsoap(&soapenv,soap_call_MXCommon__GetClientConnections(&soapenv,NULL, NULL, &DUMMY, &response));
    assert_soap_command(&soapenv,&(response.sResponse));
    printf("Current connections: %s\n\n", response.sArray.__ptr);

    writeMoreFunction(&soapenv, endpoint, nbrOfChannel);

    /* Print End programme informations */
    printf("\n\n");
    printf("+---------------------------------------------------+\n");
    printf("| Program ended                                     |\n");
    printf("+---------------------------------------------------+\n");

    return 0;
}
