#include <unistd.h>
#include <stdio.h>
#include <stdbool.h>
#include <string.h>
#include <math.h>
#include <errno.h>
#include <MSXE351x.nsmap> // this file must be included only once in the project
#include <assert.h>
#include <sys/stat.h>
#include <signal.h>
#include <time.h>
#include <pthread.h>
#include "utils.h"

/* please see utils.h */
int DUMMY = 0;

/* IP Address of the module */
char * 	ipAddress = "192.168.99.99";

struct generatorParam_t
{
	unsigned long channel;
	unsigned long dataSrc;
	unsigned long samplingRate;
	unsigned long prescaler;
	unsigned long outputType;
	unsigned long polarity;
	unsigned long numberOfCycles;
	unsigned long fixedTimeStep;
	unsigned long gateTriggerMask;
	unsigned long gateTriggerMode;
	unsigned long startMode;
	unsigned long stopState;
	unsigned long minDataNbr;

	struct UnsignedLongArray  * longData;
	struct UnsignedShortArray * shortData;
};

/*
 * Get the number of lines and the size of the csvFile
 */
void csvFilesGetNumberOfLinesAndSize(FILE * csvFile, int * numberOfLines, int * csvFileSize)
{
	char readChar = '0';

	/* go to the beginning of the file */
	fseek(csvFile, 0, SEEK_SET);

	while (!feof(csvFile))
	{
		readChar = fgetc(csvFile);
		/* get the number of lines */
		if (readChar == '\n')
		{
			*numberOfLines = *numberOfLines + 1;
		}
		/* get the file size */
		*csvFileSize = *csvFileSize + 1;
	}
	*numberOfLines = *numberOfLines - 1;
}

/*
 * Get the values from the csvFile and return the size of data to write
 */
int handleCsv(FILE * csvFile, unsigned long * values, int csvFileSize)
{
	char * strPtr = NULL;
	char * csvFileBuffer = NULL;

	int counter = 0;

	csvFileBuffer = (char *) malloc(csvFileSize);

	/* go to the beginning of the file*/
	fseek(csvFile, 0, SEEK_SET);

	/* read the file and get the data */
	do
	{
		if (fgets(csvFileBuffer, csvFileSize, csvFile) != 0)
		{
			strPtr = csvFileBuffer;
			values[counter] = atol(strPtr) << 16;
			strPtr = strstr(strPtr, ";");

			if(strPtr++ != NULL)
			{
				values[counter] = values[counter] + atol(strPtr);
				counter++;
			}
			else
			{
				printf("Error in the csv file line %i", counter);
				exit(1);
			}
		}
	}
	while(!feof(csvFile));
	free (csvFileBuffer);

	return 0;
}

/*
 * init the parameter for the generator
 */
int sampleInitParameter(struct generatorParam_t * generatorParam, int sample)
{
	/* select the source if continuous mode */
	do
	{
		printf("Enter the channel (0 -> 7) : \n");
		scanf("%lu", &generatorParam->channel);
	}
	while (generatorParam->channel < 0 || generatorParam->channel > 7);

	/* start mode */
	do
	{
		printf("Enter the start Mode : \n"
				"\t 0 : Each generator start separately\n"
				"\t 1 : All generators started at the same time\n");
		scanf("%lu", &generatorParam->startMode);
	}
	while (generatorParam->startMode < 0 || generatorParam->startMode > 1);

	/* continuous mode */
	if (sample >= 1)
	{
		/* source type */
		do
		{
			printf("Enter the source type : \n"
					"\t 1 : SOAP\n"
					"\t 2 : dataserver\n");
			scanf("%lu", &generatorParam->dataSrc);
		}
		while (generatorParam->dataSrc < 1 || generatorParam->dataSrc > 2);

		/* minimum number of data */
		do
		{
			printf("Enter the minimum number of values you want (1->65535) : \n");
			scanf("%lu",&generatorParam->minDataNbr);
		}
		while (generatorParam->minDataNbr < 1 || generatorParam->minDataNbr > 65535);
	}

	/* init sampling rate */
	do
	{
		printf("Enter the sampling rate : \n"
				"\t 0 : 20 kHz\n"
				"\t 1 : 40 kHz\n"
				"\t 2 : Synchro trigger\n"
				"\t 3 : High hardware trigger input edge\n"
				"\t 4 : Low  hardware trigger input edge\n"
				"\t 5 : High/Low hardware trigger input edge\n");
		scanf("%lu", &generatorParam->samplingRate);
	}
	while (generatorParam->samplingRate < 0 || generatorParam->samplingRate > 5);

	/* select the prescaler */
	if(generatorParam->samplingRate > 1)
	{
		do
		{
			printf("Enter the prescaler (1 to 65535) : \n");
			scanf("%lu", &generatorParam->prescaler);
		}
		while(generatorParam->prescaler < 0 || generatorParam->prescaler > 65535);
	}

	/* selection of the polarity */
	do
	{
		printf("Enter the polarity : \n"
				"\t 0 : Voltage\n"
				"\t 1 : Current\n"
				"\t 2 : System default\n");
		scanf("%lu", &generatorParam->outputType);
	}
	while (generatorParam->outputType < 0 || generatorParam->outputType > 2);

	/* selection of the polarity */
	do
	{
		printf("Enter the polarity : \n"
				"\t 0 : Unipolar\n"
				"\t 1 : Bipolar\n");
		scanf("%lu", &generatorParam->polarity);
	}
	while (generatorParam->polarity < 0 || generatorParam->polarity > 1);

	/* number of cycles */
	do
	{
		printf("Enter the number of cycles: (0 to 65535) : \n");
		scanf("%lu", &generatorParam->numberOfCycles);
	}
	while (generatorParam->numberOfCycles < 0 || generatorParam->numberOfCycles > 65535);

	/* Fixed time step */
	do
	{
		printf("Enter the fixed time step : (0 to 65535) : \n");
		scanf("%lu", &generatorParam->fixedTimeStep);
	}
	while (generatorParam->fixedTimeStep < 0 || generatorParam->fixedTimeStep > 65535);

	/* gate trigger mask */
	do
	{
		printf("Enter the trigger mask :\n"
				"\t 0 : no hardware action required to start the generator\n"
				"\t 1 : Hardware trigger required to start the generator\n"
				"\t 2 : Synchro input action required to start the generator. First trigger start the generator\n");
		scanf("%lu", &generatorParam->gateTriggerMask);
	}
	while (generatorParam->gateTriggerMask < 0 || generatorParam->gateTriggerMask > 2);

	/* gate trigger mode */
	do
	{
		printf(" Enter the trigger mode : \n "
				" \t 1 : Rising front start the generator (trigger action). \n"
				" \t 2 : Falling front start the generator (trigger action). \n"
				" \t 3 : Both front start the generator (trigger action). \n"
				" \t 5 : High level start the generator (gate action). \n"
				" \t 6 : Low level start the generator (gate action).generator\n");
		scanf("%lu", &generatorParam->gateTriggerMode);
	}
	while (generatorParam->gateTriggerMode < 1 || generatorParam->gateTriggerMode > 6);

	/* stop state */
	do
	{
		printf(" enter the stop state :\n"
				"\t 0 : The output keep the state. \n"
				"\t 1 : The output is set to 0V/0mA after write all values. \n"
				"\t 2 : The output is set to 0V/0mA after a software stop command  \n"
				"\t 3 : The output is set to 0V/0mA after write all values or a software stop command \n");
		scanf("%lu", &generatorParam->stopState);
	}
	while (generatorParam->stopState < 0 || generatorParam->stopState > 3);

	return 0;
}

/* sampleGeneratorInitSingle
 *
 * @param[in] soapEnv The current SOAP environment
 * @param[in] endPoint : SOAP endpoint
 * @param[in] nbrOfChannel : Number of channel on the board
 *
 * @return 0
 *
 */
int sampleGeneratorInitSingle(struct soap *soapEnv, char *endPoint, struct generatorParam_t * generatorParam, int sample,  FILE * csvFile)
{
	struct MSXE351x__Response response;
	unsigned long * longValues = NULL;
	unsigned short * shortValues = NULL;
	int counter = 0;
	unsigned long timeStep;

	/* stop and release the generator */
	assert_gsoap(soapEnv, soap_call_MSXE351x__GeneratorStopAndRelease(soapEnv, endPoint, NULL, 1 << generatorParam->channel, 0, 0, &response));

	/* init the parameters of the generator for single mode*/
	sampleInitParameter(generatorParam, sample);

	/* handle the csvFile if present*/
	if(csvFile != NULL)
	{
		int csvFileSize = 0;
		/* get the size and the number of lines of the file */
		csvFilesGetNumberOfLinesAndSize(csvFile, &generatorParam->longData->__size, &csvFileSize);

		/* allocation of memory for the data to write */
		longValues = (unsigned long *) malloc(generatorParam->longData->__size * sizeof(unsigned long));

		/* get the data to write*/
		handleCsv(csvFile, longValues, csvFileSize);

		/* set the pointer to the data */
		generatorParam->longData->__ptr = &longValues[1];
	}
	/* get the value given directly by the user */
	else if(csvFile == NULL)
	{
		/* check if write data without step with SOAP*/
		if (generatorParam->fixedTimeStep != 0)
		{
			// number of values
			do
			{
				printf("Enter the number of analog values you want :(0 -> 65535) \n");
				scanf("%i", &generatorParam->shortData->__size);
			}
			while (generatorParam->shortData->__size < 0 || generatorParam->shortData->__size > 65535);

			shortValues = (unsigned short *) malloc (generatorParam->shortData->__size * sizeof (unsigned short));

			for (counter = 0; counter < generatorParam->shortData->__size; counter++)
			{
				printf("Enter the %i value : \n", counter);
				scanf("%hu", (unsigned short *)&shortValues[counter]);
			}
			generatorParam->shortData->__ptr = (unsigned short *) shortValues;
		}
		else
		{
			/* number of values */
			do
			{
				printf("Enter the number of analog values you want : (0 -> 65535)");
				scanf("%i", &generatorParam->longData->__size);
			}
			while (generatorParam->longData->__size < 0 || generatorParam->longData->__size > 65535);

			longValues = (unsigned long *)malloc(generatorParam->longData->__size * sizeof(unsigned long));

			for (counter=0; counter<generatorParam->longData->__size; counter++)
			{
				printf("Enter the %i value : \n", counter);
				scanf("%lu", &longValues[counter]);

				printf("Enter the Time step for the value %i : (1 -> 65535)\n", counter);
				scanf("%lu", &timeStep);
				longValues[counter] = longValues[counter] + (timeStep<<16);
			}
			generatorParam->longData->__ptr = (unsigned long *) longValues;
		}
	}

	/* init sampling rate */
	assert_gsoap(soapEnv, soap_call_MSXE351x__GeneratorInitSamplingRate(soapEnv, endPoint, NULL, generatorParam->samplingRate, generatorParam->prescaler, 0, 0, &response));
	assert_soap_command(soapEnv, (struct DefaultResponse*) &response);

	/* generator init single */
	assert_gsoap(soapEnv, soap_call_MSXE351x__GeneratorInitSingle(soapEnv, endPoint, NULL, generatorParam->channel, generatorParam->outputType ,generatorParam->polarity, generatorParam->numberOfCycles, generatorParam->fixedTimeStep, generatorParam->gateTriggerMask, generatorParam->gateTriggerMode, generatorParam->stopState, generatorParam->longData, generatorParam->shortData, 0, 0, &response));
	printf("%i", response.iReturnValue);
	assert_soap_command(soapEnv, (struct DefaultResponse*) &response);

	/* start the generator */
	assert_gsoap(soapEnv, soap_call_MSXE351x__GeneratorStart(soapEnv, endPoint, NULL, 1 << generatorParam->channel, 0, 0, &response));
	assert_soap_command(soapEnv, (struct DefaultResponse*) &response);

   /* Release the memory allocated by the gSOAP calls */
   soap_destroy(soapEnv);
   soap_end(soapEnv);

	/* wait to stop */
	printf("\nPress a key to stop the sample\n");
	while(!kbhit());

	/* stop and release the generator */
	assert_gsoap(soapEnv, soap_call_MSXE351x__GeneratorStopAndRelease(soapEnv, endPoint, NULL, 1 << generatorParam->channel, 0, 0, &response));
	assert_soap_command(soapEnv, (struct DefaultResponse*) &response);

	return 0;
}

/* sampleGeneratorInitContinuous
 *
 * @param[in] soapEnv The current SOAP environment
 * @param[in] endPoint : SOAP endpoint
 * @param[in] clientFd : clientFd of the dataserver
 * @param[in] nbrOfChannel : Number of channel on the board
 *
 * @return 0
 */
int sampleAnalogOutputContinousMode(struct soap *soapEnv, char *endPoint, struct generatorParam_t * generatorParam, int sample, FILE * csvFile)
{
	unsigned long * longValues = NULL;
	unsigned short * shortValues = NULL;

	int timeStep = 0;
	int counter = 0;

	struct MSXE351x__Response response;
	struct MSXE351x__unsignedLongResponse ulResponse;

	/* init the parameter of the generator for continuous mode*/
	sampleInitParameter(generatorParam, sample);

	/* handle the csvFile if present*/
	if(csvFile != NULL)
	{
		int csvFileSize = 0;
		/* get the size and the number of lines of the file */
		csvFilesGetNumberOfLinesAndSize(csvFile, &generatorParam->longData->__size, &csvFileSize);

		/* alloc memory for the data to write */
		longValues = (unsigned long *) malloc(generatorParam->longData->__size * sizeof(unsigned long));

		/* get the data to write*/
		handleCsv(csvFile, longValues, csvFileSize);

		/* set the pointer to the data */
		generatorParam->longData->__ptr = &longValues[1];
	}
	/* get the value given directly by the user */
	else if(csvFile == NULL)
	{
		/* check if write data without step with SOAP*/
		if (generatorParam->fixedTimeStep != 0)
		{
			// number of values
			do
			{
				printf("Enter the number of analog values you want : (0 -> 65535)\n");
				scanf("%i", &generatorParam->shortData->__size);
			}
			while (generatorParam->shortData->__size < 0 || generatorParam->shortData->__size > 65535);

			shortValues = (unsigned short *) malloc (generatorParam->shortData->__size * sizeof (unsigned short));

			for (counter = 0; counter < generatorParam->shortData->__size; counter++)
			{
				printf("Enter the %i value : \n", counter);
				scanf("%hu", (unsigned short *)&shortValues[counter]);
			}
			generatorParam->shortData->__ptr = (unsigned short *) shortValues;
		}
		else
		{
			/* number of values */
			do
			{
				printf("Enter the number of analog values you want : (0 -> 65535)\n");
				scanf("%i", &generatorParam->longData->__size);
			}
			while (generatorParam->longData->__size < 0 || generatorParam->longData->__size > 65535);

			longValues = (unsigned long *)malloc(generatorParam->longData->__size * sizeof(unsigned long));

			for (counter=0; counter<generatorParam->longData->__size; counter++)
			{
				printf("Enter the %i value : \n", counter);
				scanf("%lu", &longValues[counter]);

				printf("Enter the Time step for the value %i :\n", counter);
				scanf("%i", &timeStep);
				longValues[counter] = longValues[counter] + (timeStep<<16);
			}
			generatorParam->longData->__ptr = (unsigned long *) longValues;
		}
	}

	/* init sampling rate */
	assert_gsoap(soapEnv, soap_call_MSXE351x__GeneratorInitSamplingRate(soapEnv, endPoint, NULL, generatorParam->samplingRate, generatorParam->prescaler, 0, 0, &response));
	assert_soap_command(soapEnv, (struct DefaultResponse*) &response);
	/* make sure the generator is not running */
	assert_gsoap(soapEnv, soap_call_MSXE351x__GeneratorStopAndRelease(soapEnv, endPoint, NULL, 1 << generatorParam->channel, 0, 0, &response));

	/* init continuous example */
	assert_gsoap(	soapEnv,
			soap_call_MSXE351x__GeneratorInitContinuous(
					soapEnv,
					endPoint,
					NULL,
					generatorParam->channel,
					generatorParam->outputType,
					generatorParam->polarity,
					generatorParam->minDataNbr,
					generatorParam->fixedTimeStep,
					generatorParam->gateTriggerMask,
					generatorParam->gateTriggerMode,
					generatorParam->startMode,
					generatorParam->stopState,
					generatorParam->dataSrc,
					0,
					0,
					&response) );
	assert_soap_command(soapEnv, (struct DefaultResponse*) &response);

	/* start the generator */
	assert_gsoap(soapEnv, soap_call_MSXE351x__GeneratorStart(soapEnv, endPoint, NULL, 1 <<  generatorParam->channel, 0, 0, &response));
	assert_soap_command(soapEnv, (struct DefaultResponse*) &response);

   /* Release the memory allocated by the gSOAP calls */
   soap_destroy(soapEnv);
   soap_end(soapEnv);

	/* select the sample */
	switch (generatorParam->dataSrc)
	{
	/* SOAP */
	case 1:
		/* wait to stop */
		printf("\nPress a key to stop the sample\n");
		/* write data with step */
		if (generatorParam->fixedTimeStep == 0)
		{
			while(!kbhit())
			{
				assert_gsoap(soapEnv, soap_call_MSXE351x__GeneratorWriteDataWithSteps(soapEnv, endPoint, NULL, generatorParam->channel, generatorParam->longData, 0, 0, &ulResponse));
				assert_soap_command(soapEnv, (struct DefaultResponse*) &ulResponse);

			   /* Release the memory allocated by the gSOAP calls */
			   soap_destroy(soapEnv);
			   soap_end(soapEnv);
			}
		}
		else
		{
			/* write data without step */
			while(!kbhit())
			{
				assert_gsoap(soapEnv, soap_call_MSXE351x__GeneratorWriteDataWithoutSteps(soapEnv, endPoint, NULL, generatorParam->channel, generatorParam->shortData, 0, 0, &ulResponse));
				assert_soap_command(soapEnv, (struct DefaultResponse*) &response);

			   /* Release the memory allocated by the gSOAP calls */
			   soap_destroy(soapEnv);
			   soap_end(soapEnv);
			}
		}
		break;
		/* dataserver */
	case 2:
	{
		int clientFd = -1;

		/* Connect to the dataserver */
		clientFd = datasrv_connect(ipAddress);

		/* wait to stop */
		printf("\nPress a key to stop the sample\n");
		/* write data with the data server */
		while ((!kbhit()))
		{
			if (generatorParam->fixedTimeStep == 0)
			{
				if ( send(clientFd, (unsigned char *)longValues, generatorParam->longData->__size * 4, 0) < 1 )
				{
					perror("recv");
					exit(1);
				}
			}
			else
			{
				if ( send(clientFd, (unsigned char *)shortValues, generatorParam->shortData->__size * 2, 0) < 1 )
				{
					perror("recv");
					exit(1);
				}
			}
		}
		break;
	}
	if(generatorParam->fixedTimeStep != 0)
	{
		free(shortValues);
	}
	else
	{
		free(longValues);
	}

	}
	/* stop and release the generator */
	assert_gsoap(soapEnv, soap_call_MSXE351x__GeneratorStopAndRelease(soapEnv, endPoint, NULL, 1 << generatorParam->channel, 0, 0, &response));
	assert_soap_command(soapEnv, (struct DefaultResponse*) &response);

	return 0;
}


/*
 *  signal handler : stops the current acquisition and cleanup the SOAP connection.
 */
static void do_signal (int sig)
{
	printf("signal received:%s\n",strsignal(sig));
	exit(1);
}

/*
 *  For all signals set the handler do_signal.
 * This allows to handle CTRL+C nicely
 * @note This function must be called after the first call to a function of the soap_call() familly.
 */
static void setup_signal(void)
{
	struct sigaction action;
	/* catch all signal */
	sigfillset(&action.sa_mask); /* all other signal will be blocked when handling the one received */
	action.sa_handler = do_signal;
	action.sa_flags = 0;
	{
		int i;
		for (i=1; i< NSIG; i++)
		{
			if (SIGKILL==i) /* this signal can not be blocked */
				continue;
			if (SIGSTOP==i) /* this signal can not be blocked */
				continue;
			sigaction(i,&action,NULL); /* manage error */
		}
	}
}

void usage(void)
{
	printf("\nThe command should be like this : ./generator <ipAddress> <sample> <csvfile.csv (optional)>\n"
			"\t ipAddress = XXX.XXX.XXX.XXX\n"
			"\t sample = \t 0 generator init single\n"
			"	\t\t 1 generator init continuous\n"
			"\t csvFile = csvfile.csv\n");
	exit(1);
}

int main(int argc, char ** argv)
{
	/* soap variables */
	char 								endPoint[30];
	struct soap							soapEnv;
	struct MXCommon__ByteArrayResponse 	response;


	FILE * csvFile = NULL;
	char * fileName;

	int sample;

	/* sample generator variables */
	struct generatorParam_t generatorParam;
	struct UnsignedShortArray shortDatas;
	struct UnsignedLongArray longDatas;

	/* Print sample information */
	printf("+---------------------------------------------------+\n");
	printf("| MSX-E351x                                         |\n");
	printf("| Generator sample 						            |\n");
	printf("+---------------------------------------------------+\n");
	printf("\n");

	if(argc < 3 || argc > 4)
	{
		usage();
	}

	/* Get the sample to run */
	sample = atoi(argv[2]);

	if (sample < 0 || sample > 1)
	{
		usage();
	}

	memset(endPoint, 0, sizeof(endPoint));

	/* this structure the current SOAP connection */
	mx_soap_init(&soapEnv);

	/* Get the ipAddress */
	ipAddress = strdup(argv[1]);

	/* generates the endpoint string
	 * SOAP "endpoints" are of the form NAME:PORT -
	 * PORT is in the case of the SOAP command server: 5555
	 */
	if( snprintf(endPoint, sizeof(endPoint), "%s:5555", ipAddress)<0)
	{
		fprintf(stderr, "error: %s:%d!\n", __FUNCTION__, __LINE__);
		exit(1);
	}

	/* get the csv file name if present */
	if (argc == 4)
	{
		fileName = strdup(argv[3]);

		/* check if csv file */
		if(strstr(fileName, ".csv"))
		{
			csvFile = fopen(fileName, "r");
		}
	}

	/* ask the module for its type */
	assert_gsoap(&soapEnv, soap_call_MXCommon__GetModuleType(&soapEnv, endPoint, NULL, &DUMMY,&response) );
	printf("Returned string: %s\n",response.sArray.__ptr);

	/* who is currently connected ? */
	assert_gsoap(&soapEnv,soap_call_MXCommon__GetClientConnections(  &soapEnv,NULL, NULL, &DUMMY,&response ));
	assert_soap_command(&soapEnv,&(response.sResponse) );

	printf("Current connections: %s\n\n", response.sArray.__ptr);

	/* parameter structure initialization */
	memset(&generatorParam, 0, sizeof(struct generatorParam_t));
	generatorParam.longData 		= &longDatas;
	generatorParam.shortData 		= &shortDatas;

	setup_signal();

	/* select the sample (single mode or continuous mode)*/
	switch (sample)
	{
	case 0:
		sampleGeneratorInitSingle(&soapEnv, endPoint, &generatorParam, sample, csvFile);
		break;
	case 1:
		sampleAnalogOutputContinousMode(&soapEnv, endPoint, &generatorParam, sample, csvFile);
		break;
	}

	/* clean the SOAP */
	mx_soap_clean(&soapEnv);

	/* close the file */
	if (csvFile != NULL)
	{
		fclose(csvFile);
	}
	/* Print End programme informations */
	printf("\n\n");
	printf("+---------------------------------------------------+\n");
	printf("| Program stopped                                   |\n");
	printf("+---------------------------------------------------+\n");


	return 				0;
}
