/** @file AnalogOutputDiagnose.c
 * 
 * AnalogOutputDiagnose: Analog output diagnose sample
 *
 * @author C. Guinot
 * @date 30.01.2007
 * 
 * @version $LastChangedRevision$
 * @version $LastChangedDate$
 * 
 * 
 * @par LICENCE
 * @verbatim
 *  Copyright (C) 2004,2005  ADDI-DATA GmbH for the source code of this module.
 *        
 *        ADDI-DATA GmbH 
 *        Airport Boulevard B210
 *        D-77836 Rheinmnster Germany 
 *        Tel: +49(0)7229/1847-0 
 *        Fax: +49(0)7229/1847-200
 *        http://www.addi-data-com 
 *        info@addi-data.com 
 *        
 *
 * @endverbatim   
 *
 * @par UPDATES                                                                                                          
 * @verbatim                                                                                                             
 *                                                                                                                       
 *  Date      : 
 *  Author        : 
 *  Description   : 
 *
 * @endverbatim     
 */ 

#include <stdio.h>
#include <conio.h>
#include <stdlib.h>


#include "..\..\..\..\Interface_Library\MSXE351x.nsmap"
#include "..\utility.h"


#define SOAP_ACTION ""

//-------------------------------------------------------------------------------------------------------

/** Write one channel value.
 * 
 * @retval 0:	No error.
 * @retval 1:	Error.
 */
int i_SAMPLE_WriteChannelValue (struct soap *ps_SoapEnv, unsigned char *pc_SOAPIPAddress, unsigned long dw_NbrOfChannel)
{
	struct MSXE351x__Response Response;
    struct MSXE351x__AnalogOutputGetStatusResponse StatusResponse;
	int i_Error = 0;
	unsigned long  ulChannel = 0;
    unsigned long  ulOutputType = 0;
    unsigned long  ulPolarity = 0;
    unsigned long  ulTriggerMask = 0;
    unsigned long  ulTriggerEdgeSelection = 0;
    unsigned long  ulTriggerCount =0;
	unsigned long  ulChannelValue = 0x7FFF;

		Response.iReturnValue = 0;
        StatusResponse.sResponse.iReturnValue = 0;
	
		printf ("\n");	

        do
        {
            printf("\n\tEnter the channel do You want to write to (0 to %d) :",dw_NbrOfChannel-1);
            scanf ("%d",&ulChannel);
        }
        while (ulChannel >= dw_NbrOfChannel);
        do
        {
            printf("\n\tEnter the channel configuration do You want to use (0:Voltage, 1:Current) :");
            scanf ("%d",&ulOutputType);
        }
        while (ulOutputType > 1);
        do
        {
            printf("\n\tEnter the channel polarity do You want to use (0:Unipolar, 1:Bipolar) :");
            scanf ("%d",&ulPolarity);
        }
        while (ulPolarity > 1);
        do
        {
            if (ulPolarity == 0)
                printf("\n\tEnter the output value to write in Hex (0 to 0x7FFF) : 0x");
            else
                printf("\n\tEnter the output value to write in Hex (0 to 0xFFFF) : 0x");
            scanf ("%lX",&ulChannelValue);
        }
        while (((ulChannelValue > 0xFFFF) && (ulPolarity == 1)) ||
               ((ulChannelValue > 0x7FFF) && (ulPolarity == 0)));
				
		i_Error = soap_call_MSXE351x__AnalogOutputWrite1Value  (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION,ulChannel,ulOutputType,ulPolarity, 
                                                                ulTriggerMask,ulTriggerEdgeSelection,ulTriggerCount,ulChannelValue,0,0,0,&Response);

		if ((i_Error == 0) && (Response.iReturnValue == 0))
		{			
		    printf ("\n\tWrite %lu on channel %lu", ulChannelValue, ulChannel);			
		} 
		else
		{
			printf("\n\tsoap_call_MSXE351x__AnalogOutputWrite1Value SOAP error: %d, Response error: %d",i_Error,Response.iReturnValue);
	        if (i_Error != 0)
	            printf("\n\tSOAP Error : %s",*soap_faultstring(ps_SoapEnv));

			return 1;	
		}
	
		return 0;
}


//-------------------------------------------------------------------------------------------------------

void main(void)
{
	int i_Error = 0;
	struct soap *ps_SoapEnv = NULL;
	struct MXCommon__ByteArrayResponse  s_ByteArrayResponse;
    struct MSXE351x__AnalogOutputDiagnosticResponse DiagnoseResponse;
    struct MSXE351x__Response Response;
	unsigned char pc_IPAddressTemp[50];
	unsigned char pc_IPAddress[50];
	unsigned char pc_SOAPIPAddress[50];
	unsigned long dw_Dummy = 0;
	unsigned long dw_NbrOfChannel = 8;
    char c_Key = 0;

	/* Print sample information */
	printf("\n+---------------------------------------------------+");
	printf("\n| MSX-E351x                                         |");
 	printf("\n| Analog output diagnose sample                     |"); 
 	printf("\n+---------------------------------------------------+");
	printf("\n");

	/* Initialise the SOAP connection */
	if (i_SOAP_Init(&ps_SoapEnv) == 0)
	{
		/* Get the IP address */
		printf("Enter the module IP Address (XXX.XXX.XXX.XXX): ");
		gets(pc_IPAddressTemp);

		/* Format and set the IP address */
		v_UTILS_ClearNullInIPAddress(pc_IPAddressTemp,pc_IPAddress);
		sprintf(pc_SOAPIPAddress,"%s:5555", pc_IPAddress);
		printf("\nSOAP IP Address: %s\n", pc_SOAPIPAddress);

		/* Get the module type */
		i_Error = soap_call_MXCommon__GetModuleType(ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, &dw_Dummy, &s_ByteArrayResponse);
		if ((i_Error == 0) && (s_ByteArrayResponse.sResponse.iReturnValue == 0))
		{
			printf("\nModule Type: %s\n", s_ByteArrayResponse.sArray.__ptr);

            printf ("\nPress a key to do an output diagnostic ...\n");

            printf ("\n\tPress ESC to exit the sample, press 'W' to write a value to an analog output \n");

					
			for ( ; ; )
			{
				if (kbhit()) 
				{
					c_Key = getch();
                    // Test if ESC
                    if (c_Key == 27)
						break;

                    // Test if W or w
                    if ((c_Key == 87) || (c_Key == 119))
                    {
						i_SAMPLE_WriteChannelValue (ps_SoapEnv, pc_SOAPIPAddress, dw_NbrOfChannel);					
                        printf("\n");
                    }
                }

            	i_Error = soap_call_MSXE351x__AnalogOutputDiagnostic  (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION,0,&DiagnoseResponse);

		        if ((i_Error == 0) && (DiagnoseResponse.sResponse.iReturnValue == 0))
		        {			
                    if ((DiagnoseResponse.ulOverTemperature != 0) ||(DiagnoseResponse.ulShortCircuitOROpenLoad != 0))
                    {
		                printf ("\nDiagnose Error occur :");	
                        if (DiagnoseResponse.ulOverTemperature != 0)		
		                    printf ("\nOver Temperature channel mask in hex: %lX",DiagnoseResponse.ulOverTemperature);			
                        if (DiagnoseResponse.ulShortCircuitOROpenLoad != 0)		
		                    printf ("\nShort-circuit / Open-load channel mask in hex: %lX",DiagnoseResponse.ulShortCircuitOROpenLoad);			

                        printf("\nPress a key to rearm the analog output diagnostic");
                        c_Key = getch();
                      	i_Error = soap_call_MSXE351x__AnalogOutputRearmDiagnostic  (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION,0,&Response);

      		            if ((i_Error == 0) && (Response.iReturnValue == 0))
	                    {			
                            printf("\nRearm OK");
                            printf("\n");
                        }
                        else
                        {
          			        printf("\nsoap_call_MSXE351x__AnalogOutputRearmDiagnostic SOAP error: %d, Response error: %d",i_Error,Response.iReturnValue);
                            if (i_Error != 0)
                                printf("\nSOAP Error : %s",*soap_faultstring(ps_SoapEnv));

                        }
                        
                        c_Key = 0;
                        
                    }
                    else
                    {
                        printf("\rDiagnose information : OK");
                    }

		        } 
		        else
		        {
			        printf("\nsoap_call_MSXE351x__AnalogOutputDiagnostic SOAP error: %d, Response error: %d",i_Error,DiagnoseResponse.sResponse.iReturnValue);
	                if (i_Error != 0)
	                    printf("\nSOAP Error : %s",*soap_faultstring(ps_SoapEnv));
            	}
				
				/* 
				 * gSOAP doesn't release automatically the memory that 
				 * it allocates to deserialize data, that's why both 
				 * following functions have to be used to release the 
				 * allocated memory.
				 */
				soap_destroy(ps_SoapEnv);
				soap_end(ps_SoapEnv);
			}
		}
		else
		{
			printf("\nsoap_call_MXCommon__GetModuleType SOAP error: %d, Response error :%d", i_Error,s_ByteArrayResponse.sResponse.iReturnValue);
            if (i_Error != 0)
                 printf("\nSOAP Error : %s",*soap_faultstring(ps_SoapEnv));
		}

		/* Close the SOAP connection */
		i_SOAP_Close (ps_SoapEnv);
	}

	/* Print End programme informations */
	printf("\n\n");
	printf("+---------------------------------------------------+\n");
	printf("| Program stopped                                   |\n");
	printf("+---------------------------------------------------+\n");
	printf("| Press any key to exit from sample                 |\n");
	printf("+---------------------------------------------------+\n");

	while (!(kbhit()));
}
