/** @file Watchdog.c
 * 
 * Watchdog: Analog output watchdog
 * o	MSXE351x__IOWatchdogInitAndStart 
 * o	MSXE351x__IOWatchdogGetStatusAndValue 
 * o	MSXE351x__IOWatchdogStopAndRelease
 * 
 *
 * @author C. Guinot
 * @date 30.01.2007
 * 
 * @version $LastChangedRevision$
 * @version $LastChangedDate$
 * 
 * 
 * @par LICENCE
 * @verbatim
 *  Copyright (C) 2004,2005  ADDI-DATA GmbH for the source code of this module.
 *        
 *        ADDI-DATA GmbH 
 *        Airport Boulevard B210
 *        D-77836 Rheinmnster Germany 
 *        Tel: +49(0)7229/1847-0 
 *        Fax: +49(0)7229/1847-200
 *        http://www.addi-data-com 
 *        info@addi-data.com 
 *        
 *
 * @endverbatim   
 *
 * @par UPDATES                                                                                                          
 * @verbatim                                                                                                             
 *                                                                                                                       
 *  Date      : 
 *  Author        : 
 *  Description   : 
 *
 * @endverbatim     
 */ 

#include <stdio.h>
#include <conio.h>
#include <stdlib.h>


#include "..\..\..\..\Interface_Library\MSXE351x.nsmap"
#include "..\utility.h"


#define SOAP_ACTION ""

//-------------------------------------------------------------------------------------------------------

/** Initialize and startthe watchdog.
 * 
 * @retval 0:	No error.
 * @retval 1:	Error.
 */
int i_SAMPLE_WatchdogInitialization (struct soap *ps_SoapEnv, unsigned char *pc_SOAPIPAddress)
{
	struct MSXE351x__Response Response;
	int i_Error = 0;
	unsigned long  ulTimeBase = 0;
	unsigned long  ulTimeValue = 0;
	
		Response.iReturnValue = 0;
        printf ("Watchdog configuration\n");	

		do 
		{				
			printf ("Time base of the watchdog delay (0 for us, 1 for ms, 2 for s): ");
			scanf ("%lu", &ulTimeBase);
            printf("\n");
		}
		while (ulTimeBase > 2);	

		do 
		{				
			printf ("Time value of the watchdog delay (1 to 65535): ");
			scanf ("%lu", &ulTimeValue);
            printf("\n");
		}
		while ( (ulTimeValue < 1) || (ulTimeValue > 0xFFFF) );		
	
		i_Error = soap_call_MSXE351x__IOWatchdogInitAndStart (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, ulTimeBase, ulTimeValue, 0, 0, &Response);
		if ((i_Error == 0) && (Response.iReturnValue == 0))
		{			
			printf ("soap_call_MSXE351x__IOWatchdogInitAndStart OK\n");			
		} 
		else
		{
			printf("soap_call_MSXE351x__IOWatchdogInitAndStart SOAP error: %d, Response error: %d\n",i_Error,Response.iReturnValue);
		    if (i_Error != 0)
		        printf("SOAP Error : %s\n",*soap_faultstring(ps_SoapEnv));
			
			return 1;	
		}
			
	
	return 0;	
}

//-------------------------------------------------------------------------------------------------------

/** Write one channel value.
 * 
 * @retval 0:	No error.
 * @retval 1:	Error.
 */
int i_SAMPLE_WriteChannelValue (struct soap *ps_SoapEnv, unsigned char *pc_SOAPIPAddress, unsigned long dw_NbrOfChannel)
{
	struct MSXE351x__Response Response;
	int i_Error = 0;
	unsigned long  ulChannel = 0;
    unsigned long  ulOutputType = 0;
    unsigned long  ulPolarity = 0;
    unsigned long  ulTriggerMask = 0;
    unsigned long  ulTriggerEdgeSelection = 0;
    unsigned long  ulTriggerCount =0;
	unsigned long  ulChannelValue = 0x7FFF;

		Response.iReturnValue = 0;
	
		printf ("\n");	
				
		i_Error = soap_call_MSXE351x__AnalogOutputWrite1Value  (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION,ulChannel,ulOutputType,ulPolarity, 
                                                                ulTriggerMask,ulTriggerEdgeSelection,ulTriggerCount,ulChannelValue,0,0,0,&Response);

		if ((i_Error == 0) && (Response.iReturnValue == 0))
		{			
			printf ("Write %lu on channel %lu\n", ulChannelValue, ulChannel);			
		} 
		else
		{
			printf("soap_call_MSXE351x__AnalogOutputWrite1Value SOAP error: %d, Response error: %d\n",i_Error,Response.iReturnValue);
	        if (i_Error != 0)
	            printf("SOAP Error : %s\n",*soap_faultstring(ps_SoapEnv));

			return 1;	
		}
	
		return 0;
}

//-------------------------------------------------------------------------------------------------------

/** Get watchdog status and value.
 * 
 * @retval 0:	No error.
 * @retval 1:	Error.
 */
int i_SAMPLE_GetWatchdogStatusAndValue (struct soap *ps_SoapEnv, unsigned char *pc_SOAPIPAddress)
{
	struct MSXE351x__IOWatchdogGetStatusAndValueResponse WatchdogStatusAndValueResponse;
	int i_Error = 0;
	unsigned long  ulChannel = 0;
	unsigned long  ulChannelValue = 1;

		WatchdogStatusAndValueResponse.sResponse.iReturnValue = 0;
	
		printf ("\n");
					
		i_Error = soap_call_MSXE351x__IOWatchdogGetStatusAndValue (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, 0, &WatchdogStatusAndValueResponse); 
		if ((i_Error == 0) && (WatchdogStatusAndValueResponse.sResponse.iReturnValue == 0))
		{			
			printf ("Watchdog status %lu value = %lu info = %lu\n", WatchdogStatusAndValueResponse.ulStatus, WatchdogStatusAndValueResponse.ulValue, WatchdogStatusAndValueResponse.ulInfo);			
		} 
		else
		{
			printf("soap_call_MSXE351x__IOWatchdogGetStatusAndValue SOAP error: %d, Response error: %d\n",i_Error,WatchdogStatusAndValueResponse.sResponse.iReturnValue);
	        if (i_Error != 0)
	            printf("SOAP Error : %s\n",*soap_faultstring(ps_SoapEnv));

			return 1;	
		}
	
		return 0;
}


//-------------------------------------------------------------------------------------------------------

/** Stop and release the watchdog.
 * 
 * @retval 0:	No error.
 * @retval 1:	Error.
 */
int i_SAMPLE_StopAndReleaseWatchdog (struct soap *ps_SoapEnv, unsigned char *pc_SOAPIPAddress)
{
	struct MSXE351x__Response Response;
	int i_Error = 0;

		printf ("\n");	
	
		Response.iReturnValue = 0;

		i_Error = soap_call_MSXE351x__IOWatchdogStopAndRelease (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, 0, &Response); 
		if ((i_Error == 0) && (Response.iReturnValue == 0))
		{			
			printf ("soap_call_MSXE351x__IOWatchdogStopAndRelease OK\n");			
		} 
		else
		{
			printf("soap_call_MSXE351x__IOWatchdogStopAndRelease SOAP error: %d, Response error: %d\n",i_Error,Response.iReturnValue);
	        if (i_Error != 0)
	            printf("SOAP Error : %s\n",*soap_faultstring(ps_SoapEnv));

			return 1;	
		}
	
		return 0;
}

//-------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------

void main(void)
{
	int i_Error = 0;
	struct soap *ps_SoapEnv = NULL;
	struct MXCommon__ByteArrayResponse  s_ByteArrayResponse;
	unsigned char pc_IPAddressTemp[50];
	unsigned char pc_IPAddress[50];
	unsigned char pc_SOAPIPAddress[50];
	unsigned long dw_Dummy = 0;
	unsigned long dw_NbrOfChannel = 8;

	/* Print sample information */
	printf("+---------------------------------------------------+\n");
	printf("| MSX-E351x                                         |\n");
 	printf("| Watchdog: Analog output watchdog sample           |\n"); 
 	printf("+---------------------------------------------------+\n");
	printf("\n");

	/* Initialise the SOAP connection */
	if (i_SOAP_Init(&ps_SoapEnv) == 0)
	{
		/* Get the IP address */
		printf("Enter the module IP Address (XXX.XXX.XXX.XXX): ");
		gets(pc_IPAddressTemp);

		/* Format and set the IP address */
		v_UTILS_ClearNullInIPAddress(pc_IPAddressTemp,pc_IPAddress);
		sprintf(pc_SOAPIPAddress,"%s:5555", pc_IPAddress);
		printf("\nSOAP IP Address: %s\n", pc_SOAPIPAddress);

		/* Get the module type */
		i_Error = soap_call_MXCommon__GetModuleType(ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, &dw_Dummy, &s_ByteArrayResponse);
		if ((i_Error == 0) && (s_ByteArrayResponse.sResponse.iReturnValue == 0))
		{
			printf("Module Type: %s\n", s_ByteArrayResponse.sArray.__ptr);

			/* Initialize and start the watchdog */
			if (i_SAMPLE_WatchdogInitialization (ps_SoapEnv, pc_SOAPIPAddress) == 0) 
			{	

				printf ("Press a key to start\n");
				printf ("Once started, press any key to set the first output to 10V (trigger the watchdog) or ESC to quit\n");
				getch();
					
				for ( ; ; )
				{
					if (i_SAMPLE_GetWatchdogStatusAndValue (ps_SoapEnv, pc_SOAPIPAddress) == 0)
					{
						/* Press any key to set all output to 1 */
						if (kbhit()) 
						{
							if (getch() == 27)
								break;

							i_SAMPLE_WriteChannelValue (ps_SoapEnv, pc_SOAPIPAddress, dw_NbrOfChannel);					
						}					
					}

					/* 
					 * gSOAP doesn't release automatically the memory that 
					 * it allocates to deserialize data, that's why both 
					 * following functions have to be used to release the 
					 * allocated memory.
					 */
					soap_destroy(ps_SoapEnv);
					soap_end(ps_SoapEnv);
				}

				/* Stop and release the watchdog */
				i_SAMPLE_StopAndReleaseWatchdog (ps_SoapEnv, pc_SOAPIPAddress);
			}

		}
		else
		{
			printf("soap_call_MXCommon__GetModuleType SOAP error: %d, Response error :%d\n", i_Error,s_ByteArrayResponse.sResponse.iReturnValue);
            if (i_Error != 0)
                 printf("SOAP Error : %s\n",*soap_faultstring(ps_SoapEnv));
		}

		/* Close the SOAP connection */
		i_SOAP_Close (ps_SoapEnv);
	}

	/* Print End programme informations */
	printf("\n\n");
	printf("+---------------------------------------------------+\n");
	printf("| Program stopped                                   |\n");
	printf("+---------------------------------------------------+\n");
	printf("| Press any key to exit from sample                 |\n");
	printf("+---------------------------------------------------+\n");

	while (!(kbhit()));
}
